require 'amrita2/template'
require 'gettext/rgettext'

module Amrita2
  class CompoundElement
    def get_erb_source
      @children.collect do |c|
        c.get_erb_source
      end.join("\n")
    end
  end

  class ErbNode
    def get_erb_source
      return @node.to_s
    end
  end

  class DynamicElement
    def get_erb_source
      @children.collect do |c|
        c.get_erb_source
      end.join("\n")
    end
  end


  class RootElement < DynamicElement
    attr_accessor :text_domain
    compile_old = self.instance_method(:compile)

    define_method(:compile) do |cg|
      cg.code("bindtextdomain(#{@text_domain.inspect})")
      compile_old.bind(self).call(cg)
    end

  end

  class TemplateText
    attr_accessor :text_domain
    def compile_for_gettext
      setup
    end

    def get_erb_source_for_gettext
      setup
      @root.get_erb_source
    end
  end

  module GetText
    class TextNodeForGetText < StaticNode
      def dynamic?
        true
      end
      
      def render_me(cg)
        cg.put_string_expression("_(#{@node.to_s.strip.inspect}) % $_")
      end
      def get_erb_source
        return ""
      end
    end
  end

  module Filters
    class GetText < Filters::Base
      def parse_node(de, node)
        if node.kind_of?(REXML::Text) and not node.kind_of?(REXML::CData)
          TextNodeForGetText.new(de, node)
        else
          super
        end
      end
    end
  end

  module GetTextParser
    include Amrita2
    include Amrita2::GetText

    module_function
    def target?(file)
      File.extname(file) == '.a2html'
    end

    def parse(file, ary)
      t = TemplateText.new(File::open(file).read) do |e, src, filters|
        filters << Filters::GetText.new
      end


      src = t.compile_for_gettext
      RubyParser.parse_lines(file, [src], ary)
      src = t.get_erb_source_for_gettext
      erb = ERB.new(src).src.split(/$/)
      RubyParser.parse_lines(file, erb, ary)
      ary.collect do |msgid, fnameandline|
        [msgid, fnameandline.gsub(/\d+$/, "-")]
      end
    end
  end
end

GetText::RGetText.add_parser(Amrita2::GetTextParser)
