
module Amrita2

  class TemplateText
    def add_macro(m)
      @macros ||= []
      @macros << m
    end

    setup_filter_proc_old = instance_method(:setup_filter_proc)
    
    define_method(:setup_filter_proc) do
      setup_filter_proc_old.bind(self).call()
      return unless @macros
      return unless @macros.size > 0
      old_proc = @filter_proc
      @filter_proc = proc do |e, name ,filters|
        filters << MacroFilter.new(*@macros)
        old_proc.call(e, name ,filters) if old_proc
      end
    end
  end
  
  module Macro
    class Base
      def initialize
        @mt = TemplateText.new(get_macro_template, :amrita_prefix=>"macro:", :inline_ruby=>true) 
      end

      def get_macro_template
        self.class.const_get(:TemplateText)
      end

      def process(de, element)
        preprocess_element(@mt, element)
      end
    end
  end

  module Filters
    class MacroFilter < Base
      include Amrita2
      include Util
      include OptionSupport
      
      def initialize(*macros)
        @macros = macros.collect do |m|
          case m
          when Class
            m.new
          else
            m
          end
        end
      end

      def filter_element(de, element)
        @macros.each do |m|
          if m.match_element(element)
            ret = m.process(de, element)
            ret = PreProcessor.new.process(ret.dup)
            ret.gsub!('<%%', '<%')
            ret.gsub!('%%>', '%>')
            doc = REXML::Document.new "<macroroot>#{ret}</macroroot>"
            element = replace_target_src(doc.root)
          else
            element.each_element do |c|
              cc = filter_element(de, c)
              element.replace_child(c, cc) if c != cc
            end
          end
        end
        element
      end

      def replace_target_src(e)
        e.attributes["am:src"] = e.attributes["target_src"] if e.attributes["target_src"]
        e.delete_attribute("target_src")
        e.each_element do |c|
          replace_target_src(c)
        end
        e
      end
      
      def element_render_code(de, cg, element, &block)
        if (element.name == 'macroroot')
          yield
        else
          super
        end
      end
    end
  end
end
