require 'amrita2/template'
require 'action_view'

module ActionView
  class Base #:nodoc:
    include Amrita2::Runtime
  end
end

module Amrita2
  module Filters
    class FormTag < Base
      def element_render_code(de, cg, element, &block)
        if element.name == "form"
          cg.put_expression %[eval("form_tag", b)]
          cg.level_up do
            yield
          end
          cg.put_constant "</form>"
        else
          super
        end
      end
    end
  end
end
module Amrita2View
  class Base
    include Amrita2
    include Amrita2::Filters
    include Amrita2::Runtime
    include Amrita2::Util

    @@compiled_amrita2_templates = {}

    def initialize( action_view )
      @action_view = action_view
    end

    def render(template, local_assigns={})
      if template.kind_of?(String)
        render_amrita(template)
      else
        @action_view.render(template, local_assigns)
      end
    end

    def setup_template(template)
      setup_template_default(template)
    end

    def setup_template_default(template)
      t = Amrita2::TemplateText.new(template, :inline_ruby)
      GetText.bindtextdomain(t.text_domain) if Object::const_defined?("GetText")
      t
    end

    def render_amrita(template)
      @@compiled_amrita2_templates[template] ||= setup_template(template)
      tmpl = @@compiled_amrita2_templates[template]
      tmpl.render_with(binding_of_view)
    end

    def binding_of_view
      @action_view.instance_eval { binding }
    end
    
  end
end

