require 'rexml/element'
require 'diff/lcs'
require 'diff/lcs/hunk'

module REXML
  class Attributes < Hash
    # redefine each_attribute for unified order
    def each_attribute
      keys.sort.each do |key|
        val = get_attribute(key)
        if val.kind_of? Attribute
          yield val
        else
          val.each_value { |atr| yield atr }
        end
      end
    end
  end    
end


module Amrita2
  module TestSupport
    def assert_equal_as_xml(expected, actual)
      assert_equal(expected.as_normalized_xml, actual.as_normalized_xml)
    rescue REXML::ParseException
      puts expected
      puts actual
      puts msg = diff_as_string(expected, actual)
      raise
    rescue
      msg = diff_as_string(expected.as_normalized_xml, actual.as_normalized_xml)
      msg = build_message "not equal as xml ", msg
      raise Test::Unit::AssertionFailedError.new(msg)
    end
    
    # copied from rspec spec/expectations/differs/default.rb
    def diff_as_string(data_old, data_new)
      context_lines = 3
      #format = :context
      format = :unified
      data_old = data_old.split(/\n/).map! { |e| e.chomp }
      data_new = data_new.split(/\n/).map! { |e| e.chomp }
      output = ""
      diffs = Diff::LCS.diff(data_old, data_new)
      return output if diffs.empty?
      oldhunk = hunk = nil  
      file_length_difference = 0
      diffs.each do |piece|
        begin
          hunk = Diff::LCS::Hunk.new(data_old, data_new, piece, context_lines,
                                     file_length_difference)
          file_length_difference = hunk.file_length_difference      
          next unless oldhunk      
          # Hunks may overlap, which is why we need to be careful when our
          # diff includes lines of context. Otherwise, we might print
          # redundant lines.
          if (context_lines > 0) and hunk.overlaps?(oldhunk)
            hunk.unshift(oldhunk)
          else
            output << oldhunk.diff(format)
          end
        ensure
          oldhunk = hunk
          output << "\n"
        end
      end  
      #Handle the last remaining hunk
      output << oldhunk.diff(format) << "\n"
    end  
    
    def should_be_samexml_as(expected)
      if self.kind_of?(NormalizedXml) && expected.kind_of?(NormalizedXml)
        super
      else
        as_normalized_xml.should_be_samexml_as(expected.as_normalized_xml)
      end
    end

    def as_normalized_xml
      case self
      when String
        t = self.gsub(/>\s+/, '>').gsub(/\s+</, '<')
        e = REXML::Document.new(t).root
        if e
          e.as_normalized_xml
        else
          NormalizedXml.new(self)
        end
      when REXML::Element
        context[:compress_whitespace] = :all
        write(a="", 0, false, true)
        a.gsub!(%r[>([ \t]*)\n*([ \t]*)<], ">#$1\n#$2<")
        NormalizedXml.new(a)
      when nil
        NormalizedXml.new("")
      else
        to_s.as_normalized_xml
      end
    end

    class NormalizedXml < String
      def samexml_as?(expected)
        self == expected
      end
    end
  end

  class TemplateText
    def test_with(value, b=nil)
      set_trace(log = "")
      yield(result = render_with(value, b))
    rescue StandardError
      p $!, value, result
      puts log
      raise
    end
  end
end

class Object
  include Amrita2::TestSupport
end


