
require 'rexml/document'
require 'amrita2'
require 'amrita2/testsupport'

include Amrita2::Filters
include Amrita2::Runtime

context "Attribute" do
  
  specify "" do
    t = Amrita2::TemplateText.new('<div><a am:src="aaa|Attr[:href]">aaa</a></div>')
    t.test_with(:aaa => {
            :href=>"http://amrita2.rubyforge.org/",
            :body=>"Amrita"
                }
                ) do |r|
      r.should_be_samexml_as '<div><a href="http://amrita2.rubyforge.org/">Amrita</a></div>'
    end
  end
  
  specify "change attribute key" do
    t = Amrita2::TemplateText.new('<div><a am:src="aaa|Attr[:href=>:url]">aaa</a></div>')
    t.test_with(:aaa => {
                    :url=>"http://amrita2.rubyforge.org/",
                    :body=>"Amrita"
                  }
                  ) do |r|
      r.should_be_samexml_as '<div><a href="http://amrita2.rubyforge.org/">Amrita</a></div>'
    end
  end
  
  specify "change body key" do
    t = Amrita2::TemplateText.new('<div><a am:src="aaa|Attr[:href=>:url, :body=>:text]">aaa</a></div>')
    t.test_with(:aaa => {
                    :url=>"http://amrita2.rubyforge.org/",
                    :text=>"Amrita"
                  }
                  ) do |r|
      r.should_be_samexml_as '<div><a href="http://amrita2.rubyforge.org/">Amrita</a></div>'
    end
  end
  
  specify "inner dynamic element" do
    text = '<div><a am:src="aaa|Attr[:href=>:url]"><span am:src="inner"/></a></div>'
    t = Amrita2::TemplateText.new(text)
    t.test_with(:aaa => {
                  :url=>"http://amrita2.rubyforge.org/",
                  :inner=>"Amrita"
                }
                ) do |r|
      r.should_be_samexml_as '<div><a href="http://amrita2.rubyforge.org/">Amrita</a></div>'
    end
    
    text = '<div><a am:src="aaa|Attr[:href=>:url]">(<em><span am:src="inner"/></em>)</a></div>'
    t = Amrita2::TemplateText.new(text)
    t.test_with(:aaa => {
                  :url=>"http://amrita2.rubyforge.org/",
                  :inner=>"Amrita"
                }
                ) do |r|
      r.should_be_samexml_as '<div><a href="http://amrita2.rubyforge.org/">(<em>Amrita</em>)</a></div>'
    end
  end
  
  specify "render with struct" do
    t = Amrita2::TemplateText.new('<div><a am:src="aaa|Attr[:href=>:url, :body=>:text]">aaa</a></div>')
    s = Struct.new(:url, :text)
    data = s.new("http://amrita2.rubyforge.org/", "Amrita")
    t.test_with(:aaa => data) do |r|
      r.should_be_samexml_as '<div><a href="http://amrita2.rubyforge.org/">Amrita</a></div>'
    end
  end
  
  specify "static and dynamic attribute" do
    t = Amrita2::TemplateText.new('<div><a class="cls1" am:src="aaa|Attr[:href]">aaa</a></div>')
    t.render_with(:aaa => {
            :href=>"http://amrita2.rubyforge.org/",
            :body=>"Amrita"
        }
    ).should_be_samexml_as '<div><a href="http://amrita2.rubyforge.org/" class="cls1">Amrita</a></div>'
  end
  
  specify "render with binding" do 
    text = '<div><a class="cls1" am:filter="Attr[:href=>:url1]">aaa</a></div>'
    t = Amrita2::TemplateText.new(text, :inline_ruby)
    @url1="http://amrita2.rubyforge.org/"
    @body="Amrita"
    t.test_with(binding) do |r|
      r.should_be_samexml_as '<div><a href="http://amrita2.rubyforge.org/" class="cls1">Amrita</a></div>'
    end
  end    

  specify "render with binding and local variables" do 
    text = '<div><a class="cls1" am:filter="Attr[:href=>:url1]">aaa</a></div>'
    t = Amrita2::TemplateText.new(text, :inline_ruby)
     url1="http://amrita2.rubyforge.org/"
     body="Amrita"
     t.test_with(binding) do |r|
       r.should_be_samexml_as '<div><a href="http://amrita2.rubyforge.org/" class="cls1">Amrita</a></div>'
     end
  end
  
  specify "pickup" do
    t = Amrita2::TemplateText.new('<div><a am:src="aaa|NVarForAttr[:url]" href="$1"><span am:src="body" /></a></div>')
    t.test_with(:aaa => {
                    :url=>"http://amrita2.rubyforge.org/",
                    :body=>"Amrita"
                  }
                  ) do |r|
      r.should_be_samexml_as '<div><a href="http://amrita2.rubyforge.org/">Amrita</a></div>'
    end
  end
  
  specify "pickup with erb" do
    t = Amrita2::TemplateText.new('<div><a am:src="aaa|NVarForAttr[:url]" href="$1"><%= $_[:body] %></a></div>')
    t.test_with(:aaa => {
                    :url=>"http://amrita2.rubyforge.org/",
                    :body=>"Amrita"
                  }
                  ) do |r|
      r.should_be_samexml_as '<div><a href="http://amrita2.rubyforge.org/">Amrita</a></div>'
    end
  end
  
  specify "Nvar with $n" do
    t = Amrita2::TemplateText.new('<a am:filter="NVar[:aaa]" href="$1" class="$$1">$$2</a>')
    t.test_with(:aaa => 'aaa') do |r|
      r.should_be_samexml_as "<a href='aaa' class='$1'>$2</a>"
    end
  end
end

