
require 'rexml/document'
require 'amrita2'
require 'amrita2/testsupport'

include Amrita2
include Amrita2::Filters
include Amrita2::Runtime


context "Minimum ERB Template" do
  include Amrita2::Runtime
   setup do
     @t = Amrita2::TemplateText.new('<span><![CDATA[ <%= 1 + 1 %> ]]></span>')
   end
  
   specify "TOPLEVEL Binding" do
     @t.test_with({}) do |result|
       result.strip.should == "2"
     end
   end
end

context "you can omit CDATA in most case" do
  include Amrita2::Runtime
   setup do
     @t = Amrita2::TemplateText.new('<span><%= 1 + 1 %></span>')
   end
  
   specify "TOPLEVEL Binding" do
     @t.test_with({}) do |result|
       result.strip.should == "2"
     end
   end
end

context "Minimum mixed Template" do
  include Amrita2::Runtime
   setup do
     @t = Amrita2::TemplateText.new('<span><span am:src="aaa" /><![CDATA[ <%= 1 + 1 %> ]]></span>')
   end
  
   specify "TOPLEVEL Binding" do
     @t.test_with(:aaa=>' 1 + 1 =') do |result|
       result.strip.should == "1 + 1 =  2"
     end
   end
end

context "Minimum ERB Template with variable" do
  include Amrita2::Runtime
  setup do
    @t = Amrita2::TemplateText.new('<span><![CDATA[ <%= a + 1 %> ]]></span>')
  end
  
  specify "TOPLEVEL Binding" do
    proc { @t.render_with({}) }.should_raise NameError
  end

  specify "set Binding" do
    a = 100
    @t.test_with(binding) do |result|
      result.strip.should == "101"
    end
  end
end

context "<%% ... %%>" do
  include Amrita2::Runtime
   specify " " do
     t = Amrita2::TemplateText.new('<%%= 1 + 1 %>')
     t.test_with({}) do |result|
       result.strip.should == "<%= 1 + 1 %>"
      t = Amrita2::TemplateText.new(result.dup)
      t.test_with({}) do |result|
        result.strip.should == "2"
      end
    end
  end
end

    
context '#{..} in erb' do
  include Amrita2::Runtime
  specify " " do
    t = Amrita2::TemplateText.new('<%= %[#{1+1}]  %>')
    t.test_with({}) do |result|
      result.strip.should == "2"
    end
  end
end

context "<%% ... %%>" do
  include Amrita2::Runtime
  specify " " do
    text = <<-END
    <span>
      <%= $_[:term1] %> + <%= $_[:term2] %> = <%= $_[:term1].to_i  +  $_[:term2].to_i %>
      <%= $_[:term1] %> - <%= $_[:term2] %> = <%= $_[:term1].to_i  -  $_[:term2].to_i %>
    </span>
    END
    
    t = Amrita2::TemplateText.new(text, :inline_ruby)
    t.test_with(:term1=>20, :term2=>30) do |result|
      result.strip.should == "20  +  30  =  50 \n       20  -  30  =  -10"
    end
  end
end

context '<%= %> in erb' do
  include Amrita2::Runtime
  specify " " do
    t = Amrita2::TemplateText.new('<%= %[<%= 1+1 %%>]  %>')
    t.set_trace(STDOUT)
    t.test_with({}) do |result|
      result.strip.should == "<%= 1+1 %>"
      t = Amrita2::TemplateText.new(result.dup)
      t.test_with({}) do |result|
        result.strip.should == "2"
      end
    end
  end
end
