
require 'rexml/element'
require 'amrita2'

context "" do
  setup do
    @g = Amrita2::CodeGenerator.new
  end

  def cg_result
    eval @g.result
  end

  def cg_result_for_class(value)
    cls = Class.new
    cls.module_eval @g.result
    cls.new.render_with(value)
  end

  specify "code" do
    @g.code "1"
    cg_result.should_equal 1
  end

  specify "put_constant" do
    @g.with_stream do
      @g.put_constant 1
    end
    cg_result.should == "1"
  end

  specify "static_element without attributes" do
    @g.with_stream do
      @g.put_rexml_node REXML::Element.new('aa')
    end
    cg_result.should == "<aa />"
  end

  specify "static_element with attributes" do
    e = REXML::Element.new('aa')
    e.add_attributes('bbb' => 'ccc', 'ddd' => 'eee')
    @g.with_stream do
      @g.put_rexml_node e
    end
    cg_result.should == "<aa bbb='ccc' ddd='eee' />"
  end

  specify "if true" do
    @g.with_stream do
      @g.if_("true") do
        @g.put_constant 2
      end
    end
    cg_result.should == "2"
  end

  specify "if false " do
    @g.with_stream do
      @g.if_("false") do
        @g.put_constant 2
      end
    end
    cg_result.should == ""
  end

  specify "case " do
    @g.with_stream do
      @g.case_("1") do
        @g.when_("String") do
          @g.put_constant 1
        end
        @g.when_("Integer") do
          @g.put_constant 2
        end
      end
    end
    cg_result.should == "2"
  end

  specify "def method" do
    @g.def_method("render_with", "value") do
      @g.put_constant("<")
      @g.code("value +=1")
      @g.put_expression("(value * 2).to_s")
      @g.put_constant(">")
    end
    cg_result_for_class(3).should == "<8>"
  end
end
