
require 'rexml/element'
require 'amrita2'

context "preprocessor" do
  include Amrita2
  specify "find CDATA" do
    t = "aaa<![CDATA[bbb"
    t =~ %r[(.*?)(<!\[CDATA\[)(.*)]
    $1.should_be == "aaa"
    $2.should_be == "<![CDATA["
    $3.should_be == "bbb"
    (t+t) =~ %r[(.*?)(<!\[CDATA\[)(.*)]
    $1.should_be == "aaa"
    $2.should_be == "<![CDATA["
    $3.should_be == "bbb" + t
    $3 =~ %r[(.*?)(<!\[CDATA\[)(.*)]
    $1.should_be == "bbbaaa"
    $2.should_be == "<![CDATA["
    $3.should_be == "bbb"
  end

  specify "preprocessor" do
    p = Amrita2::PreProcessor.new
    ret = p.parse("aaa<![")
    ret.should_be == "aaa<!["
    proc {p.parse("aaa<![CDATA[")}.should_raise(RuntimeError)
    ret = p.parse("aaa<![CDATA[bbb]]>ccc")
    ret.should_be == "aaa<![CDATA[bbb]]>ccc"
    ret = p.parse("aaa<![CDATA[bbb]]>cccAAA<![CDATA[BBB]]>CCC")
    ret.should_be == "aaa<![CDATA[bbb]]>cccAAA<![CDATA[BBB]]>CCC"
  end
  
  specify "replace <% %>" do
    p = Amrita2::PreProcessor.new
    ret = p.parse("<%aaa%>")
    ret.should_be == "<![CDATA[<%aaa%>]]>"
    ret = p.parse("a<%b%>c<%d%>e")
    ret.should_be == "a<![CDATA[<%b%>]]>c<![CDATA[<%d%>]]>e"
  end
  
  specify "replace <%= %>" do
    p = Amrita2::PreProcessor.new
    ret = p.parse("<%=aaa%>")
    ret.should_be == "<![CDATA[<%=aaa%>]]>"
    ret = p.parse("a<%b%>c<%=d%>e")
    ret.should_be == "a<![CDATA[<%b%>]]>c<![CDATA[<%=d%>]]>e"
  end
  
  specify "don't replace <%= %> in cdata" do
    p = Amrita2::PreProcessor.new
    ret = p.parse("<![CDATA[<%=aaa%>]]>")
    ret.should_be == "<![CDATA[<%=aaa%>]]>"
    ret = p.parse("a<%=b%>c<![CDATA[<%d%>]]>e")
    ret.should_be == "a<![CDATA[<%=b%>]]>c<![CDATA[<%d%>]]>e"
  end
  
  specify "not replace <%= %> in cdata in real" do
    text = <<-'END'
    <ul>
      <li am:foreach="data" am:if='$_.name =~ /amrita/'>
        <![CDATA[
          <%
            name = $_.name
            description = ($_.description or $_.name)
          %>
          <a href="http://raa.ruby-lang.org/list.rhtml?name=<%= name %>"><%= description %></a>
        ]]>
      </li>
    </ul>
    END
    ret = Amrita2::PreProcessor.new.parse(text)

    text.should_be == ret
  end
end
