
require 'rexml/document'
require 'amrita2'
require 'amrita2/testsupport'

context "Simple Template" do
  include Amrita2

  setup do
    @t = Amrita2::TemplateText.new <<EOF
<html>
<head>
<title am:src='title' />
</head>
<body>
<p class='text' am:src='text' />
</body>
</html>
EOF
  end
  
  specify "" do
    @t.render_with(:title=>'Amrita2', :text=>'Hello, Amrita2').should_be_samexml_as <<-EOF
      <html>
      <head>
        <title>Amrita2</title>
      </head>
      <body>
        <p class = "text">Hello, Amrita2</p>
      </body>
      </html>
    EOF
  end
end

# http://www.cheetahtemplate.org/examples.html
context "Cheetah's sample" do
  include Amrita2

  setup do
    @t = Amrita2::TemplateText.new <<EOF
<html>
  <head><title am:src="title" /></head>
  <body>
    <table>
      <tr am:src="clients|NVar[:surname, :firstname, :email]">
        <td>$1, $2</td>
        <td><a href="mailto:$3">$3</a></td>
     </tr>
    </table>
  </body>
</html>
EOF
  end
  
  specify "" do
    client = Struct.new(:surname, :firstname, :email)
    data = {
      :title => "Amrita2 sample",
      :clients => [
        client.new("Matsumoto", "Yukihiro", "matz@netlab.co.jp"),
        client.new("Nakajima", "Taku", "tnakajima@brain-tokyo.jp"),
      ]
    }
    @t.render_with(data).should_be_samexml_as <<-EOF
<html>
  <head><title>Amrita2 sample</title></head>
  <body>
    <table>
      <tr>
        <td>Matsumoto, Yukihiro</td>
        <td><a href='mailto:matz@netlab.co.jp'>matz@netlab.co.jp</a></td>
     </tr><tr>
        <td>Nakajima, Taku</td>
        <td><a href='mailto:tnakajima@brain-tokyo.jp'>tnakajima@brain-tokyo.jp</a></td>
     </tr>
    </table>
  </body>
</html>
EOF
  end
end

# http://d.hatena.ne.jp/tokuhirom/20070110/1168416921
context "TT's sample" do
  include Amrita2
  include Amrita2::Runtime
  setup do
    @t = Amrita2::TemplateText.new <<EOF
<table border="0">
<tr
    am:for="colors.keys.sort"
    am:v="{ :name => $_ , :color => colors[$_] }"
>
  <td
      am:v='{:bgcolor => colors["white"], :body=> $_}'
      am:filter="Attr[:bgcolor]"
  >
    <b am:src='body'><span am:src='name'/></b>
  </td>
  <td
      am:v='{:bgcolor => $_[:color] }'
      am:filter="Attr[:bgcolor]"
  >
    <span
      am:v='"&nbsp"'
      am:filter="NoSanitize | Repeat[5]"
    />
  </td>
  <td
      am:v='{:bgcolor => colors["white"] , :body=> $_[:color]}'
      am:filter="Attr[:bgcolor]"
  >
  </td>
</tr>
</table>
EOF
  end
  
  specify "" do
    @t.inline_ruby = true
    colors = {
      'white' => '#FFFFFF',
      'black' => '#000000',
    }
    @t.render_with(binding).should_be_samexml_as <<-EOF
<table border = "0">
<tr>
  <td bgcolor='#FFFFFF'>
    <b>black</b>
  </td>
  <td bgcolor='#000000'>
    &nbsp&nbsp&nbsp&nbsp&nbsp
  </td>
  <td bgcolor='#FFFFFF'>#000000</td>
</tr><tr>
  <td bgcolor='#FFFFFF'>
    <b>white</b>
  </td>
  <td bgcolor='#FFFFFF'>
    &nbsp&nbsp&nbsp&nbsp&nbsp
  </td>
  <td bgcolor='#FFFFFF'>#FFFFFF</td>
</tr>
</table>
EOF
  end
end

