
require 'rexml/document'
require 'amrita2'
require 'amrita2/testsupport'

context "Minimum Template" do
  setup do
    @t = Amrita2::TemplateText.new('<span am:src="aaa" />')
  end
  
  specify "delete span without attributes" do
    result = "Amrita2"
    @t.render_with(:aaa=>'Amrita2').should_be_samexml_as(result)
  end
end

context "Greeting" do
  setup do
    @t = Amrita2::TemplateText.new('<span am:src="greeting" />, <span am:src="name" /> !')
  end
  
  specify " " do
    @t.test_with(:greeting=>"Hello", :name=>'Amrita2') do |result|
      result.should_be_samexml_as("Hello, Amrita2 !")
    end
  end
end

context "change command attributes" do
  specify "test:src" do
    t = Amrita2::TemplateText.new('<div><span test:src="aaa" /></div>')
    t.amrita_prefix = "test:"
    expected = "<div>Amrita2</div>"
    t.test_with(:aaa=>'Amrita2') do |result|
      result.should_be_samexml_as(expected)
    end
  end
  
  specify "test:src with option for constructor" do
    t = Amrita2::TemplateText.new('<div><span test:src="aaa" /></div>', :amrita_prefix => 'test:')
    expected = "<div>Amrita2</div>"
    t.test_with(:aaa=>'Amrita2') do |result|
      result.should_be_samexml_as(expected)
    end
  end
  
  specify "use amrita_id and filter" do
    t = Amrita2::TemplateText.new(%[<div><span amrita_id="aaa" filter="Default['Amrita2'.inspect]" /></div>])
    t.amrita_src = "amrita_id"
    t.amrita_filter = "filter"
    t.test_with(:aaa=>nil) do |result|
      expected = "<div>Amrita2</div>"
      result.should_be_samexml_as(expected)
    end
  end
end

context "xhtml" do
  setup do
    @t = Amrita2::TemplateText.new <<END, :process_xhtml=>true
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en">
<head>
<title am:src="title" />
</head>
<body am:src="body"/>
</html>
END
  end
  
  specify " " do
    @t.test_with(:title=>'Amrita2', :body=>'Amrita2 can process XHTML') do |result|
      result.should_match /<\?xml/
      result.should_match /<\!DOCTYPE/
      result.should_be_samexml_as <<-END
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en">
<head>
<title>Amrita2</title>
</head>
<body>Amrita2 can process XHTML</body>
</html>
END
    end
  end
end
